#include "frame.hpp"
#include "drawing_assets.hpp"

void draw_circle(Frame *frame, int cx, int cy, int radius, COLOR outline) {
    int x = 0, y = radius;
    while (x < y) {
        int D1 = x * x + y * y - radius * radius;
        int D2 = x * x + (y - 1) * (y - 1) - radius * radius;

        if (D1 > -D2)
            y--;

        frame->set_pixel(cx + x, cy + y, outline);
        frame->set_pixel(cx + x, cy - y, outline);
        frame->set_pixel(cx + y, cy + x, outline);
        frame->set_pixel(cx + y, cy - x, outline);
        frame->set_pixel(cx - x, cy + y, outline);
        frame->set_pixel(cx - x, cy - y, outline);
        frame->set_pixel(cx - y, cy + x, outline);
        frame->set_pixel(cx - y, cy - x, outline);
        x++;
    }
}

void draw_line(Frame *frame, int x1, int y1, int x2, int y2, COLOR color) {
    int dy = y2 - y1, dx = x2 - x1;
    if (dx == 0 && dy == 0) {
        frame->set_pixel(x1, y1, color);
        return;
    }

    if (abs(dx) > abs(dy)) {
        if (x2 < x1) {
            std::swap(x1, x2);
            std::swap(y1, y2);
            dx = -dx;
            dy = -dy;
        }

        int y, dx2 = dx / 2, p = 0;
        if (dy < 0) dx2 = -dx2;
        for (int x = x1; x <= x2; x++) {
            y = (p + dx2) / dx + y1;
            p += dy;
            frame->set_pixel(x, y, color);
        }
    } else {
        if (y2 < y1) {
            std::swap(x1, x2);
            std::swap(y1, y2);
            dx = -dx;
            dy = -dy;
        }

        int x, dy2 = dy / 2, p = 0;
        if (dx < 0) dy2 = -dy2;
        for (int y = y1; y <= y2; y++) {
            x = (p + dy2) / dy + x1;
            p += dx;
            frame->set_pixel(x, y, color);
        }
    }
}