#pragma once

#include <cmath>
#include <utility>

struct COLOR {
    unsigned char red;
    unsigned char green;
    unsigned char blue;
    unsigned char alpha;

    COLOR() : red(0), green(0), blue(0), alpha(255) {}

    COLOR(unsigned char red, unsigned char green, unsigned char blue, unsigned char alpha = 255) : red(red), green(green), blue(blue), alpha(alpha) {}

};


class Frame {
    COLOR *pixels;
public:
    const int width, height;

    Frame(int _width, int _height) : width{_width}, height{_height} {
        int size = width * height;

        this->pixels = new COLOR[size];
    }

private:
    [[nodiscard]] COLOR _get_pixel(int x, int y) const {
        return this->pixels[y * this->width + x];
    }

    [[nodiscard]] COLOR &_get_pixel(int x, int y) {
        return this->pixels[y * this->width + x];
    }

public:
    void set_pixel(int x, int y, COLOR color) {
        this->_get_pixel(x, y) = color;
    }

    [[nodiscard]] COLOR get_pixel(int x, int y) const {
        return this->_get_pixel(x, y);
    }


    ~Frame() {
        delete[]this->pixels;
    }
};
